package cat.hack3.codingtests.marsrover.test;

import cat.hack3.codingtests.marsrover.api.RotableRiderRover;
import cat.hack3.codingtests.marsrover.api.cartography.Coordinates;
import cat.hack3.codingtests.marsrover.api.cartography.Direction;
import cat.hack3.codingtests.marsrover.api.cartography.PlanetMap;
import cat.hack3.codingtests.marsrover.api.commands.RoverCommand;
import cat.hack3.codingtests.marsrover.api.commands.RoverCommandFactory;

import java.util.List;
import java.util.ServiceLoader;
import java.util.stream.IntStream;

import static cat.hack3.codingtests.marsrover.api.cartography.Direction.SOUTH;
import static cat.hack3.codingtests.marsrover.api.commands.RoverCommand.Type.*;

public class CommonTests {

    public static final Direction STARTING_DIRECTION = SOUTH;

    protected RotableRiderRover rover;
    protected RoverCommand moveForwardCommand;
    protected RoverCommand moveBackwardsCommand;
    protected RoverCommand turnLeftCommand;
    protected RoverCommand turnRightCommand;

    protected void setup() {
        int mapWidth = 10;
        int mapHeight = 10;
        int latitudeStartingPoint = 2;
        int longitudeStartingPoint = 3;
        var startingCoordinates = Coordinates.of(latitudeStartingPoint, longitudeStartingPoint);

        var marsMap = getImplProviderOf(PlanetMap.Provider.class)
                .provideWith(mapHeight, mapWidth, startingCoordinates, getObstacles());

        rover = getImplProviderOf(RotableRiderRover.Provider.class)
                .provideWith(marsMap, CommonTests.STARTING_DIRECTION);

        var commandFactory = getImplProviderOf(RoverCommandFactory.Provider.class)
                .provideWith(rover);

        moveForwardCommand = commandFactory.create(MOVE_FORWARD);
        moveBackwardsCommand = commandFactory.create(MOVE_BACKWARDS);
        turnLeftCommand = commandFactory.create(TURN_LEFT);
        turnRightCommand = commandFactory.create(TURN_RIGHT);
    }

    private <T> T getImplProviderOf(Class<T> interfaceProvider) {
        return ServiceLoader
                .load(interfaceProvider)
                .findFirst()
                .orElseThrow();
    }

    protected List<Coordinates> getObstacles() {
        return List.of();
    }

    protected void repeatAction(int times, Runnable actionToRepeat) {
        IntStream.rangeClosed(1, times)
                .forEach(i -> actionToRepeat.run());
    }
}
