package cat.hack3.codingtests.marsrover.test;

import cat.hack3.codingtests.marsrover.api.cartography.Coordinates;
import cat.hack3.codingtests.marsrover.api.cartography.Direction;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static cat.hack3.codingtests.marsrover.api.cartography.Direction.*;
import static org.testng.Assert.assertEquals;

public class MarsRoverTest extends CommonTests {

    @BeforeMethod
    public void setUp() {
        setup();
    }

    @Test
    public void stepForward() {
        Direction originalDirection = rover.getCurrentDirection();

        moveForwardCommand.execute();

        assertEquals(rover.getCurrentCoordinates(), Coordinates.of(3, 3));
        assertEquals(rover.getCurrentDirection(), originalDirection);
    }

    @Test
    public void stepBackwards() {
        Direction originalDirection = rover.getCurrentDirection();

        moveBackwardsCommand.execute();
        Coordinates currentPosition = rover.getCurrentCoordinates();

        assertEquals(currentPosition, Coordinates.of(1, 3));
        assertEquals(rover.getCurrentDirection(), originalDirection);
    }

    @Test
    public void turnLef() {
        turnLeftCommand.execute();
        assertEquals(rover.getCurrentDirection(), EAST);
        turnLeftCommand.execute();
        assertEquals(rover.getCurrentDirection(), NORTH);
        turnLeftCommand.execute();
        assertEquals(rover.getCurrentDirection(), WEST);
        turnLeftCommand.execute();
        assertEquals(rover.getCurrentDirection(), SOUTH);
        turnLeftCommand.execute();
        assertEquals(rover.getCurrentDirection(), EAST);
    }

    @Test
    public void turnRight() {
        turnRightCommand.execute();
        assertEquals(rover.getCurrentDirection(), WEST);
        turnRightCommand.execute();
        assertEquals(rover.getCurrentDirection(), NORTH);
        turnRightCommand.execute();
        assertEquals(rover.getCurrentDirection(), EAST);
        turnRightCommand.execute();
        assertEquals(rover.getCurrentDirection(), SOUTH);
        turnRightCommand.execute();
        assertEquals(rover.getCurrentDirection(), WEST);
    }

    @Test
    public void loopTheWorldInLatitudeMovingForward() {
        Coordinates originalCoordinates = rover.getCurrentCoordinates();
        Direction originalDirection = rover.getCurrentDirection();

        repeatAction(10, () -> moveForwardCommand.execute());

        assertEquals(rover.getCurrentCoordinates(), originalCoordinates);
        assertEquals(rover.getCurrentDirection(), originalDirection);
    }

    @Test
    public void loopTheWorldInLatitudeMovingBackwards() {
        Coordinates originalCoordinates = rover.getCurrentCoordinates();
        Direction originalDirection = rover.getCurrentDirection();

        repeatAction(10, () -> moveBackwardsCommand.execute());

        assertEquals(rover.getCurrentCoordinates(), originalCoordinates);
        assertEquals(rover.getCurrentDirection(), originalDirection);
    }

    @Test
    public void loopTheWorldInLongitudeMovingForward() {
        Coordinates originalCoordinates = rover.getCurrentCoordinates();

        turnLeftCommand.execute();
        repeatAction(10, () -> moveForwardCommand.execute());

        assertEquals(rover.getCurrentCoordinates(), originalCoordinates);
        assertEquals(rover.getCurrentDirection(), EAST);
    }

    @Test
    public void loopTheWorldInLongitudeMovingBackwards() {
        Coordinates originalCoordinates = rover.getCurrentCoordinates();

        turnLeftCommand.execute();
        repeatAction(10, () -> moveBackwardsCommand.execute());

        assertEquals(rover.getCurrentCoordinates(), originalCoordinates);
        assertEquals(rover.getCurrentDirection(), EAST);
    }

    @Test
    public void crazyTour() {
        moveForwardCommand.execute();
        assertEquals(rover.getCurrentCoordinates(), Coordinates.of(3, 3));
        assertEquals(rover.getCurrentDirection(), SOUTH);

        turnLeftCommand.execute();
        repeatAction(23, () -> moveForwardCommand.execute());
        assertEquals(rover.getCurrentCoordinates(), Coordinates.of(3, 6));
        assertEquals(rover.getCurrentDirection(), EAST);

        repeatAction(3, () -> turnRightCommand.execute());
        repeatAction(9, () -> moveForwardCommand.execute());
        assertEquals(rover.getCurrentCoordinates(), Coordinates.of(4, 6));
        assertEquals(rover.getCurrentDirection(), NORTH);

        moveBackwardsCommand.execute();
        turnRightCommand.execute();
        moveForwardCommand.execute();
        assertEquals(rover.getCurrentCoordinates(), Coordinates.of(5, 7));
        assertEquals(rover.getCurrentDirection(), EAST);
    }

}
